@@ -87,6 +87,8 @@
 //! }
 //! ```
 
+use std::fmt;
+
 use Expr::*;
 
 /// Stores a Joker Calculus expression.
@@ -106,6 +108,32 @@ pub enum Expr {
     Jok(Box<Expr>),
 }
 
+impl fmt::Display for Expr {
+    fn fmt(&self, w: &mut fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
+        match self {
+            _0 => write!(w, "0")?,
+            _1 => write!(w, "1")?,
+            Seq(a, b) => write!(w, "{} {}", a, b)?,
+            Sel(a, b) => write!(w, "({}, {})", a, b)?,
+            Jok(a) => {
+                if let Seq(_, _) = &**a {
+                    write!(w, "?({})", a)?;
+                } else {
+                    write!(w, "?{}", a)?;
+                }
+            }
+            Not(a) => {
+                if let Seq(_, _) = &**a {
+                    write!(w, "!({})", a)?;
+                } else {
+                    write!(w, "!{}", a)?;
+                }
+            }
+        }
+        Ok(())
+    }
+}
+
 impl Expr {
     /// Evaluates an expression with Open Joker Calculus.
     pub fn eval_open(&self) -> Expr {self.eval(false)}
@@ -272,6 +300,22 @@ impl Expr {
     pub fn divergent(&self) -> bool {
         not(not(self.clone())).eval_open() != self.eval_open()
     }
+
+    /// Returns `true` if the expression is contracting with Open Joker Calculus.
+    pub fn contracting_open(&self) -> bool {self.contracting(false)}
+    /// Returns `true` if the expression is contracting with Closed Joker Calculus.
+    pub fn contracting_closed(&self) -> bool {self.contracting(true)}
+
+    /// Returns `true` if the sequence repetition of itself evaluates to itself.
+    ///
+    /// When `closed` is set to `true`,
+    /// the variant Closed Joker Calculus is used.
+    ///
+    /// When `closed` is set to `false`,
+    /// the variant Open Joker Calculus is used.
+    pub fn contracting(&self, closed: bool) -> bool {
+        seq(self.clone(), self.clone()).eval(closed) == self.eval(closed)
+    }
 }
 
 /// Platonism (terminal expression).
@@ -303,6 +347,25 @@ pub fn sel<T: Into<Expr>, U: Into<Expr>>(a: T, b: U) -> Expr {
     Sel(Box::new(a.into()), Box::new(b.into()))
 }
 
+/// Shorthand syntax for Joker Calculus expression.
+#[macro_export]
+macro_rules! jc (
+    ( 0 ) => {platonism()};
+    ( 1 ) => {seshatism()};
+    ( ? $($x:tt)+ ) => {joker(jc!($($x)+))};
+    ( ! $($x:tt)+ ) => {not(jc!($($x)+))};
+    ( 0 $($x:tt)+ ) => {platonic(jc!($($x)+))};
+    ( 1 $($x:tt)+ ) => {seshatic(jc!($($x)+))};
+    ( ( 0 , $($x:tt)+ ) ) => {sel(platonism(), jc!($($x)+))};
+    ( ( 1 , $($x:tt)+ ) ) => {sel(seshatism(), jc!($($x)+))};
+    ( ( 0 $x:tt , $($y:tt)+ ) ) => {sel(platonic(jc!($x)), jc!($($y)+))};
+    ( ( 1 $x:tt , $($y:tt)+ ) ) => {sel(seshatic(jc!($x)), jc!($($y)+))};
+    ( ( 0 ? $x:tt , $($y:tt)+ ) ) => {sel(platonic(joker(jc!($x))), jc!($($y)+))};
+    ( ( 1 ? $x:tt , $($y:tt)+ ) ) => {sel(seshatic(joker(jc!($x))), jc!($($y)+))};
+    ( ( 0 ! $x:tt , $($y:tt)+ ) ) => {sel(platonic(not(jc!($x))), jc!($($y)+))};
+    ( ( 1 ! $x:tt , $($y:tt)+ ) ) => {sel(seshatic(not(jc!($x))), jc!($($y)+))};
+);
+
 #[cfg(test)]
 mod tests {
     use super::*;
